@@ -34,33 +34,16 @@ import datetime
 import functools
 import inspect
 import itertools
-import sys
+import json
 import uuid
 
-is_simplejson = False
-if sys.version_info < (2, 7):
-    # On Python <= 2.6, json module is not C boosted, so try to use
-    # simplejson module if available
-    try:
-        import simplejson as json
-        # NOTE(mriedem): Make sure we have a new enough version of simplejson
-        # to support the namedobject_as_tuple argument. This can be removed
-        # in the Kilo release when python 2.6 support is dropped.
-        if 'namedtuple_as_object' in inspect.getargspec(json.dumps).args:
-            is_simplejson = True
-        else:
-            import json
-    except ImportError:
-        import json
-else:
-    import json
-
 from oslo_utils import encodeutils
 from oslo_utils import importutils
 from oslo_utils import timeutils
 import six
 import six.moves.xmlrpc_client as xmlrpclib
 
+ipaddress = importutils.try_import("ipaddress")
 netaddr = importutils.try_import("netaddr")
 
 _nasty_type_tests = [inspect.ismodule, inspect.isclass, inspect.ismethod,
@@ -127,6 +110,11 @@ def to_primitive(value, convert_instances=False, convert_datetime=True,
     if netaddr and isinstance(value, netaddr.IPAddress):
         return six.text_type(value)
 
+    if ipaddress and isinstance(value,
+                                (ipaddress.IPv4Address,
+                                 ipaddress.IPv6Address)):
+        return six.text_type(value)
+
     # value of itertools.count doesn't get caught by nasty_type_tests
     # and results in infinite loop when list(value) is called.
     if type(value) == itertools.count:
@@ -144,7 +132,7 @@ def to_primitive(value, convert_instances=False, convert_datetime=True,
         return 'mock'
 
     if level > max_depth:
-        return '?'
+        return None
 
     # The try block may not be necessary after the class check above,
     # but just in case ...
@@ -155,8 +143,8 @@ def to_primitive(value, convert_instances=False, convert_datetime=True,
                                       level=level,
                                       max_depth=max_depth)
         if isinstance(value, dict):
-            return dict((recursive(k), recursive(v))
-                        for k, v in six.iteritems(value))
+            return {recursive(k): recursive(v)
+                    for k, v in six.iteritems(value)}
         elif hasattr(value, 'iteritems'):
             return recursive(dict(value.iteritems()), level=level + 1)
         # Python 3 does not have iteritems
@@ -193,8 +181,6 @@ def dumps(obj, default=to_primitive, **kwargs):
     Use dump_as_bytes() to ensure that the result type is ``bytes`` on Python 2
     and Python 3.
     """
-    if is_simplejson:
-        kwargs['namedtuple_as_object'] = False
     return json.dumps(obj, default=default, **kwargs)
 
 
@@ -234,8 +220,6 @@ def dump(obj, fp, *args, **kwargs):
        The *default* parameter now uses :func:`to_primitive` by default.
     """
     default = kwargs.get('default', to_primitive)
-    if is_simplejson:
-        kwargs['namedtuple_as_object'] = False
     return json.dump(obj, fp, default=default, *args, **kwargs)
 
 
